// (C) 2001-2020 Intel Corporation. All rights reserved. 
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details. 


#ifndef __INTEL_GENERIC_SERIAL_FLASH_INTERFACE_TOP_H__
#define __INTEL_GENERIC_SERIAL_FLASH_INTERFACE_TOP_H__

#include "alt_types.h"
#include "sys/alt_flash_dev.h"
#include "sys/alt_llist.h"


#ifdef __cplusplus
extern "C"
{

#endif /* __cplusplus */


#define INTEL_GSFI_64K_BLOCK_SIZE 65536


/*
 * Description of the gsfi
 */
typedef struct intel_gsfi_dev
{
    alt_flash_dev dev;

    alt_u32 data_base;
    alt_u32 data_end;
    alt_u32 csr_base;
    alt_u32 size_in_bytes;
    alt_u32 number_of_sectors;
    alt_u32 sector_size;
    alt_u32 addr_4byte_mode;

} intel_gsfi_dev;



/*
 * Macro used by alt_sys_init.c to create data storage for driver instance
 */

#define INTEL_GENERIC_SERIAL_FLASH_INTERFACE_TOP_AVL_MEM_AVL_CSR_INSTANCE(qspi_name, avl_mem, avl_csr, qspi_dev) \
static intel_gsfi_dev qspi_dev =                                                     \
{                                                                                    \
    .dev = {                                                                         \
            .llist       = ALT_LLIST_ENTRY,                                          \
            .name        = avl_mem##_NAME,                                           \
            .write       = intel_gsfi_write,                                         \
            .read        = intel_gsfi_read,                                          \
            .get_info    = intel_gsfi_get_info,                                      \
            .erase_block = intel_gsfi_erase_block,                                   \
            .write_block = intel_gsfi_write_block,                                   \
            .base_addr   = ((void*)(avl_mem##_BASE)),                                \
            .length      = ((int)(avl_mem##_SPAN)),                                  \
           },                                                                        \
    .data_base         = ((alt_u32)(avl_mem##_BASE)),                                \
    .data_end          = ((alt_u32)(avl_mem##_BASE)) + ((alt_u32)(avl_mem##_SPAN))-1,\
    .csr_base          = ((alt_u32)(avl_csr##_BASE)),                                \
    .size_in_bytes     = ((alt_u32)(avl_mem##_SPAN)),                                \
    .number_of_sectors = ((alt_u32)(avl_mem##_SPAN) / INTEL_GSFI_64K_BLOCK_SIZE),    \
    .sector_size       = ((alt_u32) INTEL_GSFI_64K_BLOCK_SIZE),                      \
}                                                                                    \

/*
 *  Refer to Using Flash Devices in the
 *  Developing Programs Using the Hardware Abstraction Layer chapter
 *  of the Nios II Software Developer's Handbook.
*/

int intel_gsfi_read(alt_flash_dev *flash_info, int offset, void *dest_addr, int length);

int intel_gsfi_write_block(alt_flash_dev *flash_info, int block_offset, int data_offset, const void *data, int length);

int intel_gsfi_write(alt_flash_dev *flash_info, int offset, const void *src_addr, int length);

int intel_gsfi_get_info(alt_flash_fd *fd, flash_region **info, int *number_of_regions);

int intel_gsfi_erase_block(alt_flash_dev *flash_info, int block_offset);

/*
 * Initialization function
 */
alt_u32 intel_gsfi_init(intel_gsfi_dev *dev);

/*
 * alt_sys_init,c will call this macro automatically initialize the driver instance
 */
#define INTEL_GENERIC_SERIAL_FLASH_INTERFACE_TOP_INIT(name, dev) \
            intel_gsfi_init(&dev)

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INTEL_GENERIC_SERIAL_FLASH_INTERFACE_TOP_H__ */
