// ============================================================================
// Copyright (c) 2020 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Jun 29 18:22:44 2020
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
`define ENABLE_DDR4C
`define ENABLE_DDR4D
//`define ENABLE_PCIE
//`define ENABLE_QSFPDDA
//`define ENABLE_QSFPDDB
//`define ENABLE_QSFPRSV_REFCLK
//`define ENABLE_DDR4_DUAL_RANK

module DE10_Agilex_golden_top
   #(
`ifdef ENABLE_DDR4_DUAL_RANK
      parameter RANK = 1
`else
      parameter RANK = 0
`endif
   )
   (

      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_50_B3A,
      input              CLK_50_B3C,
      input              UFL_CLKIN,
      input              CLK_30M72, //reserved for future use
      input    [ 1: 0]   CLK_from_SI5397A_p,
      output   [ 1: 0]   CLK_to_SI5397A_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED_BRACKET, //LED is Low-Active
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_QSFPRSV_REFCLK
      ///////// QSFPRSV_REFCLK /////////
      input              QSFPDDRSV_REFCLK_p,
`endif /*ENABLE_QSFPRSV_REFCLK*/

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output   [RANK: 0] DDR4A_CK,
      output   [RANK: 0] DDR4A_CK_n,
      output   [RANK: 0] DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [RANK: 0] DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [RANK: 0] DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [RANK: 0] DDR4B_CK,
      output   [RANK: 0] DDR4B_CK_n,
      output   [RANK: 0] DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [RANK: 0] DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [RANK: 0] DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_DDR4C
      ///////// DDR4C /////////
      input              DDR4C_REFCLK_p,
      output   [16: 0]   DDR4C_A,
      output   [ 1: 0]   DDR4C_BA,
      output   [ 1: 0]   DDR4C_BG,
      output   [RANK: 0] DDR4C_CK,
      output   [RANK: 0] DDR4C_CK_n,
      output   [RANK: 0] DDR4C_CKE,
      inout    [ 8: 0]   DDR4C_DQS,
      inout    [ 8: 0]   DDR4C_DQS_n,
      inout    [71: 0]   DDR4C_DQ,
      inout    [ 8: 0]   DDR4C_DBI_n,
      output   [RANK: 0] DDR4C_CS_n,
      output             DDR4C_RESET_n,
      output   [RANK: 0] DDR4C_ODT,
      output             DDR4C_PAR,
      input              DDR4C_ALERT_n,
      output             DDR4C_ACT_n,
      input              DDR4C_EVENT_n,
      inout              DDR4C_SCL,
      inout              DDR4C_SDA,
      input              DDR4C_RZQ,
`endif /*ENABLE_DDR4C*/

`ifdef ENABLE_DDR4D
      ///////// DDR4D /////////
      input              DDR4D_REFCLK_p,
      output   [16: 0]   DDR4D_A,
      output   [ 1: 0]   DDR4D_BA,
      output   [ 1: 0]   DDR4D_BG,
      output   [RANK: 0] DDR4D_CK,
      output   [RANK: 0] DDR4D_CK_n,
      output   [RANK: 0] DDR4D_CKE,
      inout    [ 8: 0]   DDR4D_DQS,
      inout    [ 8: 0]   DDR4D_DQS_n,
      inout    [71: 0]   DDR4D_DQ,
      inout    [ 8: 0]   DDR4D_DBI_n,
      output   [RANK: 0] DDR4D_CS_n,
      output             DDR4D_RESET_n,
      output   [RANK: 0] DDR4D_ODT,
      output             DDR4D_PAR,
      input              DDR4D_ALERT_n,
      output             DDR4D_ACT_n,
      input              DDR4D_EVENT_n,
      inout              DDR4D_SCL,
      inout              DDR4D_SDA,
      input              DDR4D_RZQ,
`endif /*ENABLE_DDR4D*/

      ///////// SI5397A /////////
      inout              SI5397A_I2C_SCL,
      inout              SI5397A_I2C_SDA,
      input              SI5397A_LOL_A,
      input              SI5397A_LOL_B,
      input              SI5397A_LOL_C,
      input              SI5397A_LOL_D,
      input              SI5397A_LOS_XAXB,
      output             SI5397A_OE_n,
      output             SI5397A_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_p,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      output   [15: 0]   PCIE_TX_p,
      output   [15: 0]   PCIE_TX_n,
      input    [15: 0]   PCIE_RX_p,
      input    [15: 0]   PCIE_RX_n,
      input    [ 1: 0]   PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFPDDA
      ///////// QSFPDDA /////////
      output   [ 7: 0]   QSFPDDA_TX_p,
      output   [ 7: 0]   QSFPDDA_TX_n,
      input    [ 7: 0]   QSFPDDA_RX_p,
      input    [ 7: 0]   QSFPDDA_RX_n,
      input              QSFPDDA_REFCLK_p,
      output             QSFPDDA_INITMODE,
      input              QSFPDDA_INTERRUPT_n,
      input              QSFPDDA_MOD_PRS_n,
      output             QSFPDDA_MOD_SEL_n,
      output             QSFPDDA_RST_n,
      inout              QSFPDDA_SCL,
      inout              QSFPDDA_SDA,
`else
      input              QSFPDDA_REFCLK_p,
`endif /*ENABLE_QSFPDDA*/

`ifdef ENABLE_QSFPDDB
      ///////// QSFPDDB /////////
      output   [ 7: 0]   QSFPDDB_TX_p,
      output   [ 7: 0]   QSFPDDB_TX_n,
      input    [ 7: 0]   QSFPDDB_RX_p,
      input    [ 7: 0]   QSFPDDB_RX_n,
      input              QSFPDDB_REFCLK_p,
      output             QSFPDDB_INITMODE,
      input              QSFPDDB_INTERRUPT_n,
      input              QSFPDDB_MOD_PRS_n,
      output             QSFPDDB_MOD_SEL_n,
      output             QSFPDDB_RST_n,
      inout              QSFPDDB_SCL,
      inout              QSFPDDB_SDA,
`endif /*ENABLE_QSFPDDB*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

      ///////// EXP /////////
      input              EXP_EN,

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);





//=======================================================
//  REG/WIRE declarations
//=======================================================
/////////////////////////// DDR4 x4 Verify ////////////////////////////////
wire user_clk_a; // clock for test controllers
wire user_clk_b; // clock for test controllers
wire user_clk_c; // clock for test controllers
wire user_clk_d; // clock for test controllers

/// test status ..
//DDR4 Verify (A)
parameter DDR4A_AVALON_ADDR_W = 27;
parameter DDR4A_AVALON_DATA_W = 512;
parameter DDR4A_BYTE_ENABLE = 72;
wire ddr4a_test_pass/*synthesis keep*/;
wire ddr4a_test_fail/*synthesis keep*/;
wire ddr4a_test_complete/*synthesis keep*/;
wire ddr4a_local_cal_success/*synthesis keep*/;
wire ddr4a_local_cal_fail/*synthesis keep*/;

/// test status ..
//DDR4 Verify (B)
parameter DDR4B_AVALON_ADDR_W = 27;
parameter DDR4B_AVALON_DATA_W = 512;
parameter DDR4B_BYTE_ENABLE = 72;
wire ddr4b_test_pass/*synthesis keep*/;
wire ddr4b_test_fail/*synthesis keep*/;
wire ddr4b_test_complete/*synthesis keep*/;
wire ddr4b_local_cal_success/*synthesis keep*/;
wire ddr4b_local_cal_fail/*synthesis keep*/;

//DDR4 Verify (C)
parameter DDR4C_AVALON_ADDR_W = 27;
parameter DDR4C_AVALON_DATA_W = 512;
parameter DDR4C_BYTE_ENABLE = 72;
wire ddr4c_test_pass/*synthesis keep*/;
wire ddr4c_test_fail/*synthesis keep*/;
wire ddr4c_test_complete/*synthesis keep*/;
wire ddr4c_local_cal_success/*synthesis keep*/;
wire ddr4c_local_cal_fail/*synthesis keep*/;

/// test status ..
//DDR4 Verify (D)
parameter DDR4D_AVALON_ADDR_W = 27;
parameter DDR4D_AVALON_DATA_W = 512;
parameter DDR4D_BYTE_ENABLE = 72;
wire ddr4d_test_pass/*synthesis keep*/;
wire ddr4d_test_fail/*synthesis keep*/;
wire ddr4d_test_complete/*synthesis keep*/;
wire ddr4d_local_cal_success/*synthesis keep*/;
wire ddr4d_local_cal_fail/*synthesis keep*/;

wire global_reset;

//=======================================================
//  Structural coding
//=======================================================

assign SI5340A_OE_n = 1'b0;
assign SI5340A_RST_n = 1'b1;


edge_detector edge_detector_Inst(

	.iCLK(CLK_50_B3C),
	.iRST_n(S_global_resetn),
	.iIn(!BUTTON[1]),
	.oFallING_EDGE(),
	.oRISING_EDGE(),
	.oDEBOUNCE_OUT(global_reset)
);

wire ninit_done;
reset_release reset_release (
	.ninit_done(ninit_done)
	);

assign S_global_resetn = !ninit_done;


wire calbusAB_clk;
wire calbusCD_clk;


    DDR4_Cal DDR4_AB_Cal_Inst (
        .calbus_read_0          (calbusA_read),          //  output,     width = 1,   emif_calbus_0.calbus_read
        .calbus_write_0         (calbusA_write),         //  output,     width = 1,                .calbus_write
        .calbus_address_0       (calbusA_address),       //  output,    width = 20,                .calbus_address
        .calbus_wdata_0         (calbusA_wdata),         //  output,    width = 32,                .calbus_wdata
        .calbus_rdata_0         (calbusA_rdata),         //   input,    width = 32,                .calbus_rdata
        .calbus_seq_param_tbl_0 (calbusA_seq_param_tbl), //   input,  width = 4096,                .calbus_seq_param_tbl

        .calbus_read_1          (calbusB_read),          //  output,     width = 1,   emif_calbus_1.calbus_read
        .calbus_write_1         (calbusB_write),         //  output,     width = 1,                .calbus_write
        .calbus_address_1       (calbusB_address),       //  output,    width = 20,                .calbus_address
        .calbus_wdata_1         (calbusB_wdata),         //  output,    width = 32,                .calbus_wdata
        .calbus_rdata_1         (calbusB_rdata),         //   input,    width = 32,                .calbus_rdata
        .calbus_seq_param_tbl_1 (calbusB_seq_param_tbl), //   input,  width = 4096,                .calbus_seq_param_tbl
        .calbus_clk             (calbusAB_clk)              //  output,     width = 1, emif_calbus_clk.clk
    );


    DDR4_Cal DDR4_CD_Cal_Inst (
        .calbus_read_0          (calbusC_read),          //  output,     width = 1,   emif_calbus_0.calbus_read
        .calbus_write_0         (calbusC_write),         //  output,     width = 1,                .calbus_write
        .calbus_address_0       (calbusC_address),       //  output,    width = 20,                .calbus_address
        .calbus_wdata_0         (calbusC_wdata),         //  output,    width = 32,                .calbus_wdata
        .calbus_rdata_0         (calbusC_rdata),         //   input,    width = 32,                .calbus_rdata
        .calbus_seq_param_tbl_0 (calbusC_seq_param_tbl), //   input,  width = 4096,                .calbus_seq_param_tbl

        .calbus_read_1          (calbusD_read),          //  output,     width = 1,   emif_calbus_1.calbus_read
        .calbus_write_1         (calbusD_write),         //  output,     width = 1,                .calbus_write
        .calbus_address_1       (calbusD_address),       //  output,    width = 20,                .calbus_address
        .calbus_wdata_1         (calbusD_wdata),         //  output,    width = 32,                .calbus_wdata
        .calbus_rdata_1         (calbusD_rdata),         //   input,    width = 32,                .calbus_rdata
        .calbus_seq_param_tbl_1 (calbusD_seq_param_tbl), //   input,  width = 4096,                .calbus_seq_param_tbl
        .calbus_clk             (calbusCD_clk)              //  output,     width = 1, emif_calbus_clk.clk
    );



`ifdef ENABLE_DDR4A
///////////////// DDR4(A) Calibration ///////////////////
wire										calbusA_read;
wire										calbusA_write;
wire	[19:0]							calbusA_address;
wire	[31:0]							calbusA_wdata;
wire	[31:0]							calbusA_rdata;
wire	[4095:0]							calbusA_seq_param_tbl;

    DDR4_Singal DDR4A_Inst (
        .local_reset_req      (global_reset),      //   input,     width = 1,    local_reset_req.local_reset_req
        .local_reset_done     (ddr4a_reset_done),     //  output,     width = 1, local_reset_status.local_reset_done
        .pll_ref_clk          (DDR4A_REFCLK_p),          //   input,     width = 1,        pll_ref_clk.clk
        .oct_rzqin            (DDR4A_RZQ),            //   input,     width = 1,                oct.oct_rzqin
        .mem_ck               (DDR4A_CK),               //  output,     width = 1,                mem.mem_ck
        .mem_ck_n             (DDR4A_CK_n),             //  output,     width = 1,                   .mem_ck_n
        .mem_a                (DDR4A_A),                //  output,    width = 17,                   .mem_a
        .mem_act_n            (DDR4A_ACT_n),            //  output,     width = 1,                   .mem_act_n
        .mem_ba               (DDR4A_BA),               //  output,     width = 2,                   .mem_ba
        .mem_bg               (DDR4A_BG),               //  output,     width = 2,                   .mem_bg
        .mem_cke              (DDR4A_CKE),              //  output,     width = 1,                   .mem_cke
        .mem_cs_n             (DDR4A_CS_n),             //  output,     width = 1,                   .mem_cs_n
        .mem_odt              (DDR4A_ODT),              //  output,     width = 1,                   .mem_odt
        .mem_reset_n          (DDR4A_RESET_n),          //  output,     width = 1,                   .mem_reset_n
        .mem_par              (DDR4A_PAR),              //  output,     width = 1,                   .mem_par
        .mem_alert_n          (DDR4A_ALERT_n),          //   input,     width = 1,                   .mem_alert_n
        .mem_dqs              (DDR4A_DQS),              //   inout,     width = 9,                   .mem_dqs
        .mem_dqs_n            (DDR4A_DQS_n),            //   inout,     width = 9,                   .mem_dqs_n
        .mem_dq               (DDR4A_DQ),               //   inout,    width = 72,                   .mem_dq
        .mem_dbi_n            (DDR4A_DBI_n),            //   inout,     width = 9,                   .mem_dbi_n
        .local_cal_success    (ddr4a_local_cal_success),    //  output,     width = 1,             status.local_cal_success
        .local_cal_fail       (ddr4a_local_cal_fail),       //  output,     width = 1,                   .local_cal_fail
        .emif_usr_reset_n     (ddr4a_emif_usr_reset_n),     //  output,     width = 1,   emif_usr_reset_n.reset_n
        .emif_usr_clk         (user_clk_a),         //  output,     width = 1,       emif_usr_clk.clk
        .amm_ready_0          (ddr4a_avl_ready),          //  output,     width = 1,         ctrl_amm_0.waitrequest_n
        .amm_read_0           (ddr4a_avl_read_req),           //   input,     width = 1,                   .read
        .amm_write_0          (ddr4a_avl_write_req),          //   input,     width = 1,                   .write
        .amm_address_0        (ddr4a_avl_addr),        //   input,    width = 27,                   .address
        .amm_readdata_0       (ddr4a_avl_rdata),       //  output,   width = 512,                   .readdata
        .amm_writedata_0      (ddr4a_avl_wdata),      //   input,   width = 512,                   .writedata
        .amm_burstcount_0     (ddr4a_avl_size),     //   input,     width = 7,                   .burstcount
        .amm_byteenable_0     ({DDR4A_BYTE_ENABLE{1'b1}}),     //   input,    width = 72,                   .byteenable
        .amm_readdatavalid_0  (ddr4a_avl_rdata_valid),  //  output,     width = 1,                   .readdatavalid
        .calbus_read          (calbusA_read),          //   input,     width = 1,        emif_calbus.calbus_read
        .calbus_write         (calbusA_write),         //   input,     width = 1,                   .calbus_write
        .calbus_address       (calbusA_address),       //   input,    width = 20,                   .calbus_address
        .calbus_wdata         (calbusA_wdata),         //   input,    width = 32,                   .calbus_wdata
        .calbus_rdata         (calbusA_rdata),         //  output,    width = 32,                   .calbus_rdata
        .calbus_seq_param_tbl (calbusA_seq_param_tbl), //  output,  width = 4096,                   .calbus_seq_param_tbl
        .calbus_clk           (calbusAB_clk)            //   input,     width = 1,    emif_calbus_clk.clk
    );

/////////////////// DDR4(A) Test ///////////////////
wire         							ddr4a_avl_ready;                  //          avl.waitrequest_n
wire [DDR4A_AVALON_ADDR_W-1:0]	ddr4a_avl_addr;                   //             .address
wire         							ddr4a_avl_rdata_valid;            //             .readdatavalid
wire [DDR4A_AVALON_DATA_W-1:0]	ddr4a_avl_rdata;                  //             .readdata
wire [DDR4A_AVALON_DATA_W-1:0]	ddr4a_avl_wdata;                  //             .writedata
wire         							ddr4a_avl_read_req;               //             .read
wire         							ddr4a_avl_write_req;              //             .write
wire [6:0]   							ddr4a_avl_size;                   //             .burstcount
wire 			 							ddr4a_emif_usr_reset_n;
wire 			 							ddr4a_reset_done;


Avalon_bus_RW_Test DDR4A_Verify(
		    .iCLK(user_clk_a),
		    .iRST_n(ddr4a_emif_usr_reset_n ),
		    .insert_error(BUTTON[0]),

		    .local_init_done(ddr4a_local_cal_success),
		    .avl_waitrequest_n(ddr4a_avl_ready),
		    .avl_address(ddr4a_avl_addr),
		    .avl_readdatavalid(ddr4a_avl_rdata_valid),
		    .avl_readdata(ddr4a_avl_rdata),
		    .avl_writedata(ddr4a_avl_wdata),
		    .avl_read(ddr4a_avl_read_req),
		    .avl_write(ddr4a_avl_write_req),
		    .avl_size(ddr4a_avl_size),

		    .drv_status_pass(ddr4a_test_pass),
		    .drv_status_fail(ddr4a_test_fail),
		    .drv_status_test_complete(ddr4a_test_complete)

);
defparam DDR4A_Verify.ADDR_W = DDR4A_AVALON_ADDR_W;
defparam DDR4A_Verify.DATA_W = DDR4A_AVALON_DATA_W;

`endif

`ifdef ENABLE_DDR4B
///////////////// DDR4(B) Calibration ///////////////////
wire										calbusB_read;
wire										calbusB_write;
wire	[19:0]							calbusB_address;
wire	[31:0]							calbusB_wdata;
wire	[31:0]							calbusB_rdata;
wire	[4095:0]							calbusB_seq_param_tbl;

    DDR4_Singal DDR4B_Inst (
        .local_reset_req      (global_reset),      //   input,     width = 1,    local_reset_req.local_reset_req
        .local_reset_done     (ddr4b_reset_done),     //  output,     width = 1, local_reset_status.local_reset_done
        .pll_ref_clk          (DDR4B_REFCLK_p),          //   input,     width = 1,        pll_ref_clk.clk
        .oct_rzqin            (DDR4B_RZQ),            //   input,     width = 1,                oct.oct_rzqin
        .mem_ck               (DDR4B_CK),               //  output,     width = 1,                mem.mem_ck
        .mem_ck_n             (DDR4B_CK_n),             //  output,     width = 1,                   .mem_ck_n
        .mem_a                (DDR4B_A),                //  output,    width = 17,                   .mem_a
        .mem_act_n            (DDR4B_ACT_n),            //  output,     width = 1,                   .mem_act_n
        .mem_ba               (DDR4B_BA),               //  output,     width = 2,                   .mem_ba
        .mem_bg               (DDR4B_BG),               //  output,     width = 2,                   .mem_bg
        .mem_cke              (DDR4B_CKE),              //  output,     width = 1,                   .mem_cke
        .mem_cs_n             (DDR4B_CS_n),             //  output,     width = 1,                   .mem_cs_n
        .mem_odt              (DDR4B_ODT),              //  output,     width = 1,                   .mem_odt
        .mem_reset_n          (DDR4B_RESET_n),          //  output,     width = 1,                   .mem_reset_n
        .mem_par              (DDR4B_PAR),              //  output,     width = 1,                   .mem_par
        .mem_alert_n          (DDR4B_ALERT_n),          //   input,     width = 1,                   .mem_alert_n
        .mem_dqs              (DDR4B_DQS),              //   inout,     width = 9,                   .mem_dqs
        .mem_dqs_n            (DDR4B_DQS_n),            //   inout,     width = 9,                   .mem_dqs_n
        .mem_dq               (DDR4B_DQ),               //   inout,    width = 72,                   .mem_dq
        .mem_dbi_n            (DDR4B_DBI_n),            //   inout,     width = 9,                   .mem_dbi_n
        .local_cal_success    (ddr4b_local_cal_success),    //  output,     width = 1,             status.local_cal_success
        .local_cal_fail       (ddr4b_local_cal_fail),       //  output,     width = 1,                   .local_cal_fail
        .emif_usr_reset_n     (ddr4b_emif_usr_reset_n),     //  output,     width = 1,   emif_usr_reset_n.reset_n
        .emif_usr_clk         (user_clk_b),         //  output,     width = 1,       emif_usr_clk.clk
        .amm_ready_0          (ddr4b_avl_ready),          //  output,     width = 1,         ctrl_amm_0.waitrequest_n
        .amm_read_0           (ddr4b_avl_read_req),           //   input,     width = 1,                   .read
        .amm_write_0          (ddr4b_avl_write_req),          //   input,     width = 1,                   .write
        .amm_address_0        (ddr4b_avl_addr),        //   input,    width = 27,                   .address
        .amm_readdata_0       (ddr4b_avl_rdata),       //  output,   width = 512,                   .readdata
        .amm_writedata_0      (ddr4b_avl_wdata),      //   input,   width = 512,                   .writedata
        .amm_burstcount_0     (ddr4b_avl_size),     //   input,     width = 7,                   .burstcount
        .amm_byteenable_0     ({DDR4B_BYTE_ENABLE{1'b1}}),     //   input,    width = 72,                   .byteenable
        .amm_readdatavalid_0  (ddr4b_avl_rdata_valid),  //  output,     width = 1,                   .readdatavalid
        .calbus_read          (calbusB_read),          //   input,     width = 1,        emif_calbus.calbus_read
        .calbus_write         (calbusB_write),         //   input,     width = 1,                   .calbus_write
        .calbus_address       (calbusB_address),       //   input,    width = 20,                   .calbus_address
        .calbus_wdata         (calbusB_wdata),         //   input,    width = 32,                   .calbus_wdata
        .calbus_rdata         (calbusB_rdata),         //  output,    width = 32,                   .calbus_rdata
        .calbus_seq_param_tbl (calbusB_seq_param_tbl), //  output,  width = 4096,                   .calbus_seq_param_tbl
        .calbus_clk           (calbusAB_clk)            //   input,     width = 1,    emif_calbus_clk.clk
    );

/////////////////// DDR4(B) Test ///////////////////
wire         							ddr4b_avl_ready;                  //          avl.waitrequest_n
wire [DDR4B_AVALON_ADDR_W-1:0]	ddr4b_avl_addr;                   //             .address
wire         							ddr4b_avl_rdata_valid;            //             .readdatavalid
wire [DDR4B_AVALON_DATA_W-1:0]	ddr4b_avl_rdata;                  //             .readdata
wire [DDR4B_AVALON_DATA_W-1:0]	ddr4b_avl_wdata;                  //             .writedata
wire         							ddr4b_avl_read_req;               //             .read
wire         							ddr4b_avl_write_req;              //             .write
wire [6:0]   							ddr4b_avl_size;                   //             .burstcount
wire 			 							ddr4b_emif_usr_reset_n;
wire 			 							ddr4b_reset_done;


Avalon_bus_RW_Test DDR4B_Verify(
		    .iCLK(user_clk_b),
		    .iRST_n(ddr4b_emif_usr_reset_n ),
		    .insert_error(BUTTON[0]),

		    .local_init_done(ddr4b_local_cal_success),
		    .avl_waitrequest_n(ddr4b_avl_ready),
		    .avl_address(ddr4b_avl_addr),
		    .avl_readdatavalid(ddr4b_avl_rdata_valid),
		    .avl_readdata(ddr4b_avl_rdata),
		    .avl_writedata(ddr4b_avl_wdata),
		    .avl_read(ddr4b_avl_read_req),
		    .avl_write(ddr4b_avl_write_req),
		    .avl_size(ddr4b_avl_size),

		    .drv_status_pass(ddr4b_test_pass),
		    .drv_status_fail(ddr4b_test_fail),
		    .drv_status_test_complete(ddr4b_test_complete)

);
defparam DDR4B_Verify.ADDR_W = DDR4B_AVALON_ADDR_W;
defparam DDR4B_Verify.DATA_W = DDR4B_AVALON_DATA_W;

`endif

`ifdef ENABLE_DDR4C
/////////////////// DDR4(C) Calibration ///////////////////
wire										calbusC_read;
wire										calbusC_write;
wire	[19:0]							calbusC_address;
wire	[31:0]							calbusC_wdata;
wire	[31:0]							calbusC_rdata;
wire	[4095:0]							calbusC_seq_param_tbl;



    DDR4_Singal DDR4C_Inst (
        .local_reset_req      (global_reset),      //   input,     width = 1,    local_reset_req.local_reset_req
        .local_reset_done     (ddr4c_reset_done),     //  output,     width = 1, local_reset_status.local_reset_done
        .pll_ref_clk          (DDR4C_REFCLK_p),          //   input,     width = 1,        pll_ref_clk.clk
        .oct_rzqin            (DDR4C_RZQ),            //   input,     width = 1,                oct.oct_rzqin
        .mem_ck               (DDR4C_CK),               //  output,     width = 1,                mem.mem_ck
        .mem_ck_n             (DDR4C_CK_n),             //  output,     width = 1,                   .mem_ck_n
        .mem_a                (DDR4C_A),                //  output,    width = 17,                   .mem_a
        .mem_act_n            (DDR4C_ACT_n),            //  output,     width = 1,                   .mem_act_n
        .mem_ba               (DDR4C_BA),               //  output,     width = 2,                   .mem_ba
        .mem_bg               (DDR4C_BG),               //  output,     width = 2,                   .mem_bg
        .mem_cke              (DDR4C_CKE),              //  output,     width = 1,                   .mem_cke
        .mem_cs_n             (DDR4C_CS_n),             //  output,     width = 1,                   .mem_cs_n
        .mem_odt              (DDR4C_ODT),              //  output,     width = 1,                   .mem_odt
        .mem_reset_n          (DDR4C_RESET_n),          //  output,     width = 1,                   .mem_reset_n
        .mem_par              (DDR4C_PAR),              //  output,     width = 1,                   .mem_par
        .mem_alert_n          (DDR4C_ALERT_n),          //   input,     width = 1,                   .mem_alert_n
        .mem_dqs              (DDR4C_DQS),              //   inout,     width = 9,                   .mem_dqs
        .mem_dqs_n            (DDR4C_DQS_n),            //   inout,     width = 9,                   .mem_dqs_n
        .mem_dq               (DDR4C_DQ),               //   inout,    width = 72,                   .mem_dq
        .mem_dbi_n            (DDR4C_DBI_n),            //   inout,     width = 9,                   .mem_dbi_n
        .local_cal_success    (ddr4c_local_cal_success),    //  output,     width = 1,             status.local_cal_success
        .local_cal_fail       (ddr4c_local_cal_fail),       //  output,     width = 1,                   .local_cal_fail
        .emif_usr_reset_n     (ddr4c_emif_usr_reset_n),     //  output,     width = 1,   emif_usr_reset_n.reset_n
        .emif_usr_clk         (user_clk_c),         //  output,     width = 1,       emif_usr_clk.clk
        .amm_ready_0          (ddr4c_avl_ready),          //  output,     width = 1,         ctrl_amm_0.waitrequest_n
        .amm_read_0           (ddr4c_avl_read_req),           //   input,     width = 1,                   .read
        .amm_write_0          (ddr4c_avl_write_req),          //   input,     width = 1,                   .write
        .amm_address_0        (ddr4c_avl_addr),        //   input,    width = 27,                   .address
        .amm_readdata_0       (ddr4c_avl_rdata),       //  output,   width = 512,                   .readdata
        .amm_writedata_0      (ddr4c_avl_wdata),      //   input,   width = 512,                   .writedata
        .amm_burstcount_0     (ddr4c_avl_size),     //   input,     width = 7,                   .burstcount
        .amm_byteenable_0     ({DDR4C_BYTE_ENABLE{1'b1}}),     //   input,    width = 72,                   .byteenable
        .amm_readdatavalid_0  (ddr4c_avl_rdata_valid),  //  output,     width = 1,                   .readdatavalid
        .calbus_read          (calbusC_read),          //   input,     width = 1,        emif_calbus.calbus_read
        .calbus_write         (calbusC_write),         //   input,     width = 1,                   .calbus_write
        .calbus_address       (calbusC_address),       //   input,    width = 20,                   .calbus_address
        .calbus_wdata         (calbusC_wdata),         //   input,    width = 32,                   .calbus_wdata
        .calbus_rdata         (calbusC_rdata),         //  output,    width = 32,                   .calbus_rdata
        .calbus_seq_param_tbl (calbusC_seq_param_tbl), //  output,  width = 4096,                   .calbus_seq_param_tbl
        .calbus_clk           (calbusCD_clk)            //   input,     width = 1,    emif_calbus_clk.clk
    );

/////////////////// DDR4(C) Test ///////////////////
wire         							ddr4c_avl_ready;                  //          avl.waitrequest_n
wire [DDR4C_AVALON_ADDR_W-1:0]	ddr4c_avl_addr;                   //             .address
wire         							ddr4c_avl_rdata_valid;            //             .readdatavalid
wire [DDR4C_AVALON_DATA_W-1:0]	ddr4c_avl_rdata;                  //             .readdata
wire [DDR4C_AVALON_DATA_W-1:0]	ddr4c_avl_wdata;                  //             .writedata
wire         							ddr4c_avl_read_req;               //             .read
wire         							ddr4c_avl_write_req;              //             .write
wire [6:0]   							ddr4c_avl_size;                   //             .burstcount
wire 			 							ddr4c_emif_usr_reset_n;
wire 			 							ddr4c_reset_done;


Avalon_bus_RW_Test DDR4C_Verify(
		    .iCLK(user_clk_c),
		    .iRST_n(ddr4c_emif_usr_reset_n ),
		    .insert_error(BUTTON[0]),

		    .local_init_done(ddr4c_local_cal_success),
		    .avl_waitrequest_n(ddr4c_avl_ready),
		    .avl_address(ddr4c_avl_addr),
		    .avl_readdatavalid(ddr4c_avl_rdata_valid),
		    .avl_readdata(ddr4c_avl_rdata),
		    .avl_writedata(ddr4c_avl_wdata),
		    .avl_read(ddr4c_avl_read_req),
		    .avl_write(ddr4c_avl_write_req),
		    .avl_size(ddr4c_avl_size),

		    .drv_status_pass(ddr4c_test_pass),
		    .drv_status_fail(ddr4c_test_fail),
		    .drv_status_test_complete(ddr4c_test_complete)

);
defparam DDR4C_Verify.ADDR_W = DDR4C_AVALON_ADDR_W;
defparam DDR4C_Verify.DATA_W = DDR4C_AVALON_DATA_W;

`endif

`ifdef ENABLE_DDR4D
/////////////////// DDR4(D) Calibration ///////////////////
wire										calbusD_read;
wire										calbusD_write;
wire	[19:0]							calbusD_address;
wire	[31:0]							calbusD_wdata;
wire	[31:0]							calbusD_rdata;
wire	[4095:0]							calbusD_seq_param_tbl;


    DDR4_Singal DDR4D_Inst (
        .local_reset_req      (global_reset),      //   input,     width = 1,    local_reset_req.local_reset_req
        .local_reset_done     (ddr4d_reset_done),     //  output,     width = 1, local_reset_status.local_reset_done
        .pll_ref_clk          (DDR4D_REFCLK_p),          //   input,     width = 1,        pll_ref_clk.clk
        .oct_rzqin            (DDR4D_RZQ),            //   input,     width = 1,                oct.oct_rzqin
        .mem_ck               (DDR4D_CK),               //  output,     width = 1,                mem.mem_ck
        .mem_ck_n             (DDR4D_CK_n),             //  output,     width = 1,                   .mem_ck_n
        .mem_a                (DDR4D_A),                //  output,    width = 17,                   .mem_a
        .mem_act_n            (DDR4D_ACT_n),            //  output,     width = 1,                   .mem_act_n
        .mem_ba               (DDR4D_BA),               //  output,     width = 2,                   .mem_ba
        .mem_bg               (DDR4D_BG),               //  output,     width = 2,                   .mem_bg
        .mem_cke              (DDR4D_CKE),              //  output,     width = 1,                   .mem_cke
        .mem_cs_n             (DDR4D_CS_n),             //  output,     width = 1,                   .mem_cs_n
        .mem_odt              (DDR4D_ODT),              //  output,     width = 1,                   .mem_odt
        .mem_reset_n          (DDR4D_RESET_n),          //  output,     width = 1,                   .mem_reset_n
        .mem_par              (DDR4D_PAR),              //  output,     width = 1,                   .mem_par
        .mem_alert_n          (DDR4D_ALERT_n),          //   input,     width = 1,                   .mem_alert_n
        .mem_dqs              (DDR4D_DQS),              //   inout,     width = 9,                   .mem_dqs
        .mem_dqs_n            (DDR4D_DQS_n),            //   inout,     width = 9,                   .mem_dqs_n
        .mem_dq               (DDR4D_DQ),               //   inout,    width = 72,                   .mem_dq
        .mem_dbi_n            (DDR4D_DBI_n),            //   inout,     width = 9,                   .mem_dbi_n
        .local_cal_success    (ddr4d_local_cal_success),    //  output,     width = 1,             status.local_cal_success
        .local_cal_fail       (ddr4d_local_cal_fail),       //  output,     width = 1,                   .local_cal_fail
        .emif_usr_reset_n     (ddr4d_emif_usr_reset_n),     //  output,     width = 1,   emif_usr_reset_n.reset_n
        .emif_usr_clk         (user_clk_d),         //  output,     width = 1,       emif_usr_clk.clk
        .amm_ready_0          (ddr4d_avl_ready),          //  output,     width = 1,         ctrl_amm_0.waitrequest_n
        .amm_read_0           (ddr4d_avl_read_req),           //   input,     width = 1,                   .read
        .amm_write_0          (ddr4d_avl_write_req),          //   input,     width = 1,                   .write
        .amm_address_0        (ddr4d_avl_addr),        //   input,    width = 27,                   .address
        .amm_readdata_0       (ddr4d_avl_rdata),       //  output,   width = 512,                   .readdata
        .amm_writedata_0      (ddr4d_avl_wdata),      //   input,   width = 512,                   .writedata
        .amm_burstcount_0     (ddr4d_avl_size),     //   input,     width = 7,                   .burstcount
        .amm_byteenable_0     ({DDR4D_BYTE_ENABLE{1'b1}}),     //   input,    width = 72,                   .byteenable
        .amm_readdatavalid_0  (ddr4d_avl_rdata_valid),  //  output,     width = 1,                   .readdatavalid
        .calbus_read          (calbusD_read),          //   input,     width = 1,        emif_calbus.calbus_read
        .calbus_write         (calbusD_write),         //   input,     width = 1,                   .calbus_write
        .calbus_address       (calbusD_address),       //   input,    width = 20,                   .calbus_address
        .calbus_wdata         (calbusD_wdata),         //   input,    width = 32,                   .calbus_wdata
        .calbus_rdata         (calbusD_rdata),         //  output,    width = 32,                   .calbus_rdata
        .calbus_seq_param_tbl (calbusD_seq_param_tbl), //  output,  width = 4096,                   .calbus_seq_param_tbl
        .calbus_clk           (calbusCD_clk)            //   input,     width = 1,    emif_calbus_clk.clk
    );

/////////////////// DDR4(D) Test ///////////////////
wire         							ddr4d_avl_ready;                  //          avl.waitrequest_n
wire [DDR4D_AVALON_ADDR_W-1:0]	ddr4d_avl_addr;                   //             .address
wire         							ddr4d_avl_rdata_valid;            //             .readdatavalid
wire [DDR4D_AVALON_DATA_W-1:0]	ddr4d_avl_rdata;                  //             .readdata
wire [DDR4D_AVALON_DATA_W-1:0]	ddr4d_avl_wdata;                  //             .writedata
wire         							ddr4d_avl_read_req;               //             .read
wire         							ddr4d_avl_write_req;              //             .write
wire [6:0]   							ddr4d_avl_size;                   //             .burstcount
wire 			 							ddr4d_emif_usr_reset_n;
wire 			 							ddr4d_reset_done;


Avalon_bus_RW_Test DDR4D_Verify(
		    .iCLK(user_clk_d),
		    .iRST_n(ddr4d_emif_usr_reset_n ),
		    .insert_error(BUTTON[0]),

		    .local_init_done(ddr4d_local_cal_success),
		    .avl_waitrequest_n(ddr4d_avl_ready),
		    .avl_address(ddr4d_avl_addr),
		    .avl_readdatavalid(ddr4d_avl_rdata_valid),
		    .avl_readdata(ddr4d_avl_rdata),
		    .avl_writedata(ddr4d_avl_wdata),
		    .avl_read(ddr4d_avl_read_req),
		    .avl_write(ddr4d_avl_write_req),
		    .avl_size(ddr4d_avl_size),

		    .drv_status_pass(ddr4d_test_pass),
		    .drv_status_fail(ddr4d_test_fail),
		    .drv_status_test_complete(ddr4d_test_complete)

);
defparam DDR4D_Verify.ADDR_W = DDR4D_AVALON_ADDR_W;
defparam DDR4D_Verify.DATA_W = DDR4D_AVALON_DATA_W;
`endif
//
heart_beat  heart_beat_0 ( .CLK(CLK_50_B3C ),     .CLK_FREQ  (50_000_000) , . CK_1HZ (heart_led) );

assign LED[0] =  (ddr4a_local_cal_success) ?~(ddr4a_test_pass & ddr4a_test_complete):heart_led;
assign LED[1] =  (ddr4b_local_cal_success) ?~(ddr4b_test_pass & ddr4b_test_complete):heart_led;
assign LED[2] =  (ddr4c_local_cal_success) ?~(ddr4c_test_pass & ddr4c_test_complete):heart_led;
assign LED[3] =  (ddr4d_local_cal_success) ?~(ddr4d_test_pass & ddr4d_test_complete):heart_led;


endmodule
