// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



`timescale 1 ps / 1 ps

`define XGMII_START     8'hFB
`define XGMII_TERMINATE 8'hFD

//`define RX_SM_IDLE      1'b0
//`define RX_SM_TRANSMIT  1'b1

`define S0_OFFSET				0
`define S4_OFFSET				72/2

module alt_ehipc3_fm_sl_xgmii_sink (
	clock,
	// reset,
        i_reset_sync_tx_clk,
        i_reset_sync_rx_clk,
	xgmii_tx_clk,
	xgmii_tx,
	xgmii_rx_clk,
	xgmii_rx,
	test_done,
	rx_ready,
	fifo_full,
	fifo_empty,
	read_stored_data,
	checker_pass,
	rx_mismatch,
	idle_state,
	xgmii_tx_d_mon,
	xgmii_tx_c_mon,
	xgmii_rx_d_mon,
	xgmii_rx_c_mon,
	o_match_count,
	o_mismatch_count,
	rx_valid
	
); // module alt_ehipc3_fm_sl_xgmii_sink

	input              clock;
        // input              reset;
        input              i_reset_sync_tx_clk;
        input              i_reset_sync_rx_clk;
	input              xgmii_tx_clk;
	input       [71:0] xgmii_tx;
	input              xgmii_rx_clk;
	input       [71:0] xgmii_rx;
	input              rx_ready;
	input wire  [63:0] xgmii_rx_d_mon;
	input wire   [7:0] xgmii_rx_c_mon;
	input wire  [63:0] xgmii_tx_d_mon;
	input wire   [7:0] xgmii_tx_c_mon;
	input              test_done;
	input wire         rx_valid;
	output wire        fifo_full;
	output wire        fifo_empty;
	output      [71:0] read_stored_data;
	output reg         checker_pass;
	output reg         rx_mismatch = 0;
	output reg  [13:0] o_match_count;
	output reg  [13:0] o_mismatch_count;
	output reg				 idle_state=1;

	wire         [71:0] xgmii_rx_compare;
	wire               rdreq;
	reg                packet_write;
	reg                packet_read;
	reg         [13:0] match_count;
	reg         [13:0] mismatch_count;
	reg          [2:0] next_state;
        reg          [2:0] current_state;
	reg                dly_rx_valid_1c;
	reg                dly_rx_valid_2c;
	reg                dly_rx_valid_3c;
	reg                match_count_reg;
	reg                mismatch_count_reg;

	reg [11:0]incr_match_count;
	reg [11:0]incr_mismatch_count;
	reg [11:0]incr_match_count_reg;
	reg [11:0]incr_mismatch_count_reg;
	reg set_match_count_reg;
	reg set_mismatch_count_reg;
	reg set_checker_pass;
	reg set_rx_mismatch;
	reg set_idle_state;
	reg set_packet_read;

//   
// Output register for mismatch_count and match_count
//   
always @ (posedge xgmii_rx_clk) begin
   if (i_reset_sync_rx_clk) begin  
      o_match_count    <= 14'h0;
      o_mismatch_count <= 14'h0;
   end else begin
      o_match_count    <= match_count;
      o_mismatch_count <= mismatch_count;
   end
end

always @(posedge xgmii_tx_clk) begin
	if (i_reset_sync_tx_clk) begin	
		packet_write   <= 0;
		// checker_pass   <= 0;
		// match_count    <= 0;
		// mismatch_count <= 0;
		// packet_read    <= 0;
		// idle_state     <= 1;
	end else begin
		if (rx_ready) begin
			if ((xgmii_tx_d_mon[7:0] == `XGMII_START) && (xgmii_tx_c_mon[0] == 1'b1) && ~fifo_full) begin
				packet_write <= 1;
			end else if (xgmii_tx_d_mon[31:24] == `XGMII_TERMINATE && xgmii_tx_c_mon[3] == 1'b1) begin
				packet_write <= 0;
			end
		end
	end
end

// delay rx_valid by two clock cycles
always @(posedge xgmii_tx_clk) begin
   dly_rx_valid_1c <= rx_valid;
   dly_rx_valid_2c <= dly_rx_valid_1c;
   dly_rx_valid_3c <= dly_rx_valid_2c;
end

//   
// FSM states declaration   
//   
localparam RX_SM_IDLE          = 0;
localparam RX_SM_TRANSMIT      = 1;

//   
// FSM
//  
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin
      current_state <= RX_SM_IDLE;
   end else begin
      current_state <= next_state;		
   end
end

//
// Next state logic
//  	
always @ (current_state or match_count or mismatch_count or match_count_reg or mismatch_count_reg or 
         test_done or xgmii_rx_d_mon or xgmii_rx_c_mon or fifo_empty or dly_rx_valid_3c or xgmii_rx_compare or read_stored_data) 
begin
   next_state = current_state;

   incr_match_count       = 12'b0;
   incr_mismatch_count    = 12'b0;
   set_match_count_reg    = 1'b0;
   set_mismatch_count_reg = 1'b0;
   set_checker_pass       = 1'b0;
   set_rx_mismatch        = 1'b0;
   set_idle_state         = 1'b1;
   set_packet_read        = 1'b0;

   case (current_state)
      RX_SM_IDLE: begin
         set_idle_state  = 1'b1;
         set_packet_read = 1'b0;
         if (match_count[7] == 1'b1) begin
            set_match_count_reg = 1'b1;
         end else begin
            set_match_count_reg = 1'b0;
         end

         if (mismatch_count < 1) begin
            set_mismatch_count_reg = 1'b0;
         end else begin
            set_mismatch_count_reg = 1'b1;
         end

         if ((test_done==1'b1) && (match_count_reg==1'b1) && (mismatch_count_reg==1'b0)) begin
            set_checker_pass = 1'b1;
         end else if ((test_done==1'b1) && (mismatch_count_reg == 1'b1)) begin
            set_rx_mismatch = 1'b1;
         end

         if ((xgmii_rx_d_mon[7:0] == `XGMII_START) && (xgmii_rx_c_mon[0] == 1'b1) && ~fifo_empty && dly_rx_valid_3c) begin
            set_packet_read = 1'b1; 
            next_state = RX_SM_TRANSMIT;
         end
      end

      RX_SM_TRANSMIT: begin
         set_idle_state = 1'b0;
         set_packet_read = 1'b1; 
         if (xgmii_rx_compare[5:0] == read_stored_data[5:0]) begin
            incr_match_count[0] = 1'b1;
         end else begin
            incr_mismatch_count[0] = 1'b1;
         end
         if (xgmii_rx_compare[11:6] == read_stored_data[11:6]) begin
            incr_match_count[1] = 1'b1;
         end else begin
            incr_mismatch_count[1] = 1'b1;
         end
         if (xgmii_rx_compare[17:12] == read_stored_data[17:12]) begin
            incr_match_count[2] = 1'b1;
         end else begin
            incr_mismatch_count[2] = 1'b1;
         end
         if (xgmii_rx_compare[23:18] == read_stored_data[23:18]) begin
            incr_match_count[3] = 1'b1;
         end else begin
            incr_mismatch_count[3] = 1'b1;
         end
         if (xgmii_rx_compare[29:24] == read_stored_data[29:24]) begin
            incr_match_count[4] = 1'b1;
         end else begin
            incr_mismatch_count[4] = 1'b1;
         end
         if (xgmii_rx_compare[35:30] == read_stored_data[35:30]) begin
            incr_match_count[5] = 1'b1;
         end else begin
            incr_mismatch_count[5] = 1'b1;
         end
         if (xgmii_rx_compare[41:36] == read_stored_data[41:36]) begin
            incr_match_count[6] = 1'b1;
         end else begin
            incr_mismatch_count[6] = 1'b1;
         end
         if (xgmii_rx_compare[47:42] == read_stored_data[47:42]) begin
            incr_match_count[7] = 1'b1;
         end else begin
            incr_mismatch_count[7] = 1'b1;
         end
         if (xgmii_rx_compare[53:48] == read_stored_data[53:48]) begin
            incr_match_count[8] = 1'b1;
         end else begin
            incr_mismatch_count[8] = 1'b1;
         end
         if (xgmii_rx_compare[59:54] == read_stored_data[59:54]) begin
            incr_match_count[9] = 1'b1;
         end else begin
            incr_mismatch_count[9] = 1'b1;
         end
         if (xgmii_rx_compare[65:60] == read_stored_data[65:60]) begin
            incr_match_count[10] = 1'b1;
         end else begin
            incr_mismatch_count[10] = 1'b1;
         end
         if (xgmii_rx_compare[71:66] == read_stored_data[71:66]) begin
            incr_match_count[11] = 1'b1;
         end else begin
            incr_mismatch_count[11] = 1'b1;
         end

         if (xgmii_rx_d_mon[31:24] == `XGMII_TERMINATE && xgmii_rx_c_mon[3] == 1'b1 && dly_rx_valid_3c) begin
            set_packet_read = 1'b0; 
            next_state <= RX_SM_IDLE;
         end
      end

   endcase // case (current_state)
end

//Register for incr match and mismatch count
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
     incr_match_count_reg    <= 12'b0;
     incr_mismatch_count_reg <= 12'b0;
   end else begin
     incr_match_count_reg    <= incr_match_count;
     incr_mismatch_count_reg <= incr_mismatch_count;
   end
end

//   
// Output register for match_count
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      match_count <= 1'b0;
   end else begin
      if (&incr_match_count_reg) begin
         match_count <= match_count + 1'b1;
      end
   end
end

//   
// Output register for mismatch_count
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      mismatch_count <= 1'b0;
   end else begin
      if (|incr_mismatch_count_reg) begin
         mismatch_count <= mismatch_count + 1'b1;
      end
   end
end

//   
// Output register for match_count_reg
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      match_count_reg <= 1'b0;
   end else begin
      if (set_match_count_reg) begin
         match_count_reg <= 1'b1;
      end else begin
         match_count_reg <= 1'b0;
      end
   end
end

//   
// Output register for mismatch_count_reg
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      mismatch_count_reg <= 1'b0;
   end else begin
      if (set_mismatch_count_reg) begin
         mismatch_count_reg <= 1'b1;
      end else begin
         mismatch_count_reg <= 1'b0;
      end
   end
end

//   
// Output register for checker_pass
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      checker_pass <= 1'b0;
   end else begin
      if (set_checker_pass) begin
         checker_pass <= 1'b1;
      end else begin
         checker_pass <= 1'b0;
      end
   end
end

//   
// Output register for rx_mismatch
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      rx_mismatch <= 1'b0;
   end else begin
      if (set_rx_mismatch) begin
         rx_mismatch <= 1'b1;
      end else begin
         rx_mismatch <= 1'b0;
      end
   end
end

//   
// Output register for idle_state
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      idle_state <= 1'b1;
   end else begin
      if (set_idle_state) begin
         idle_state <= 1'b1;
      end else begin
         idle_state <= 1'b0;
      end
   end
end

//   
// Output register for packet_read
//   
always @ (posedge xgmii_rx_clk or negedge rx_ready)
begin
   if (~rx_ready) begin  
      packet_read <= 1'b0;
   end else begin
      if (set_packet_read) begin
         packet_read <= 1'b1;
      end else begin
         packet_read <= 1'b0;
      end
   end
end

assign xgmii_rx_compare = {xgmii_rx_d_mon,xgmii_rx_c_mon};
assign rdreq = packet_read & dly_rx_valid_3c;
	alt_ehipc3_fm_sl_xgmii_checker_fifo alt_ehipc3_fm_sl_xgmii_checker_fifo_inst (
		.aclr(~rx_ready),
		.data({xgmii_tx_d_mon,xgmii_tx_c_mon}),
		.rdclk(xgmii_rx_clk),
		.rdreq(rdreq),
		.wrclk(xgmii_tx_clk),
		.wrreq(packet_write),
		.q(read_stored_data),
		.rdempty(fifo_empty),
		.wrfull(fifo_full)
	);
	
endmodule
