module si5397a_freq_prameter_selector (

input         [3:0]          iPLL_OUT_FREQ_SEL,

//output   reg  [23:0]         RX_REG,
output   reg                 OUTX_OE,
output   reg  [43:0]         NX_NUM,
output   reg  [31:0]         NX_DEN       

);
//------XCVR

// XA(48MHz) --> DSPLLx free run(no clean) --> OUT2/6(7) --> FPGA XCVR bank
`define XCVR_644M53125     4'h0// 23589522196   
`define XCVR_625M          4'h1// 24326694765   
`define XCVR_322M265625    4'h2// 47179044392   
`define XCVR_312M5         4'h3// 48653389529   
`define XCVR_307M2         4'h4// 49492787200   
`define XCVR_250M          4'h5// 60816736911   
`define XCVR_184M32        4'h6// 82487978667  
`define XCVR_156M25        4'h7// 97306779058 //default  
`define XCVR_153M6         4'h8// 98985574400   
`define XCVR_125M          4'h9// 121633473823  
`define XCVR_100M          4'hA// 152041842278  
`define XCVR_50M           4'hB// 304083684557 





// XA/INx --> PLL clean --> OUT0/4 --> FPGA IO bank
`define IO_50M             4'h0  // XA 48MHz --> DSPLL free-run 50MHZ --> OUT (default)
`define IO_IN0             4'h1  // IN0 (from FPGA, assume 30.72MHz)--> DSPLLx clean --> OUT
`define IO_IN1             4'h2  // IN1 (from FPGA, assume 31.25MHz)--> DSPLLx clean --> OUT  
`define IO_IN2             4'h3  // IN2 (30.72MHz)--> DSPLLx clean --> OUT  
`define IO_IN3             4'h4  // IN3 (from UFL, assume 10MHz)--> DSPLLx clean --> OUT   


always @(*)
  begin
     case(iPLL_OUT_FREQ_SEL[3:0])
	  
      4'h0 :   //644.53125MHz
        begin
          OUTX_OE <= 1'b1 ;
          NX_NUM  <= 44'd23589522196 ;
        end
      4'h1 :   //625MHz
        begin
          OUTX_OE <= 1'b1 ;
          NX_NUM  <= 44'd24326694765 ;
        end
      4'h2 :   //322.265625MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd47179044392 ;

        end
      4'h3 :   //312.5Hz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd48653389529;
        end                     
      4'h4 :   //307.2MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd49492787200 ;
        end                
      4'h5 :   //250MHZ
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd60816736911;
        end               
      4'h6 :   //184.32MHZ
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd82487978667;
        end               
      4'h7 :   //156.25MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd97306779058 ;
        end               
      4'h8 :   //153.6MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd98985574400;
        end               		  	  
      4'h9 :   //125MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd121633473823 ;
        end               		  	  
      4'ha :   //100MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd152041842278;
        end               		  	  
      4'hb :   //50MHz
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd304083684557 ;
        end               		  	  
      4'hc :   // inclock =outclock 
        begin
          OUTX_OE <= 1'b1;
          NX_NUM  <= 44'd2147483648;
        end               		  	  
		  
               
      default :   //100Mhz
        begin
          OUTX_OE = 1'b1;
          NX_NUM  = 44'd152253235200;
        end                         
      endcase
  end
  

endmodule
